;;; cd entry subclauses.
(define (cd-number n)
  (tag-data 'cd-number (as-string n)))

(define (cd-artist a)
  (tag-data 'cd-artist (as-string a)))

(define (cd-title t)
  (tag-data 'cd-title (as-string t)))

(define (cd-playing-time min sec)
  (tag-data 'cd-playing-time 
    (string-append (as-string min) ":"
    (as-string sec))))


;; Present a cd entry
(define (cd-entry . subentries)
 (let ((number ; @a
         (get-defaulted-prop 'cd-number subentries "?"))
       (artist 
         (get-defaulted-prop 'cd-artist subentries "No artist"))
       (title
         (get-defaulted-prop 'cd-title subentries "No title"))
       (play-time 
         (get-defaulted-prop 'cd-playing-time subentries 
                             "Unknown playing time"))
      )
  (table 'border 2   ; @b
    (tr (tdb "Number") (td number))
    (tr (tdb "Artist") (td artist))
    (tr (tdb "Title") (td title))
    (tr (tdb "Playing time") (td play-time))
  )))

;; Present a cd archive
(define (cd-archive . cd-entries)
  (html
   (head (title page-title))

   (body
     (h1 page-title)         ; @d
     
     (map spacy cd-entries)  ; @e
   )
  )
)



;;; Other definitions:

; Tag string-data with tag 
(define (tag-data tag string-data)
  (list (as-symbol tag) string-data))

; Get property key from the property list.
; In this context a property list is a structure like ((k1 v1) (k2 v2) ... ),
; thus in reality an association list with (k v) instead of (k . v) entries.
; The third parameter is the default in case key is not present in prop-list
(define (get-defaulted-prop key prop-list default)
 (let ((res-list (defaulted-get key prop-list default)))
   (if (and res-list (pair? res-list))
       (car res-list)
       default)))

(define (spacy x) (list x (p)))

(define (tdb x) (td (b x)))
